//conversion Lingvo DLS to text source for Dict (http://www.free-dict.narod.ru)

/***************************************************************************
 *                                                                         *
 *   Dedicated to Corinna Wender, a beautiful italian                      *
 *   and her visit in Russia, Perm                                         *
 *   Author: Leonid "PhoeniX" Ponomarev                                    *
 *   mailto: l_px@mail.ru                                                  *
 *   Jabber: phoenix@2on.net                                               *
 *           ptiss@livejournal.com                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   Modified by: Aleksey Soloviev (solovav@yandex.ru)       			   *
 *   Added:                                                                *
 *    - config file                                                        *
 *    - check utf-8 BOM                                                    *
 *    - some other code                                                    *     
 *                                                                         *
 ***************************************************************************/

/*
 * to build this under Qt3 you must replace toAscii() to ascii()
 */
#include <qstring.h> /* http://www.trolltech.com/developer/downloads/qt/ */ 
#include <qregexp.h>

#include <json/json.h> /* http://sourceforge.net/projects/jsoncpp/ */

#include <time.h>

#include <string>
#include <iostream>
#include <fstream>
using namespace std;


void do_replace(const Json::Value &replace_array, QString &qstr)
{
	for(unsigned i = 0; i < replace_array.size(); ++i)
	{
		string pattern = replace_array[i][0u].asString();
		string repl = replace_array[i][1].asString();
		bool is_regexp = replace_array[i][2].asString() == "regexp";
		bool regexp_minimal = replace_array[i][3].asBool();

		if(!is_regexp)
			qstr.replace(pattern.c_str(), repl.c_str());
		else
		{
			QRegExp regexp;
			regexp.setMinimal(regexp_minimal);
			regexp.setPattern(pattern.c_str());
			qstr.replace(regexp, repl.c_str());
		}
	}
}

int main(int argc, char * argv[])
{
	if (argc < 3 || argc > 4)
	{
		cout << "Wrong param count\n";
		cout << "usage:\n" <<
				"qdsl2zdt src.dsl(utf-8) dst.txt [config.cfg]\n";
		
		return 1;
 	}

	ifstream infile(argv[1], ios_base::binary);
	if(!infile)
	{	
		cout << "can't read" << argv[1] << "\n"; 
		exit(EXIT_FAILURE);
	}

	unsigned bom = 0;
	infile.read((char *)&bom, 3);
	if(bom != 0xbfbbef)
		infile.seekg(0, ios_base::beg);

	ofstream outfile(argv[2]);
	if(!outfile)
	{	
		cout << "can't read" << argv[2] << "\n"; 
		exit(EXIT_FAILURE);
	}

	string cfg_file_name;
	if(argc == 4)
		cfg_file_name = argv[3];
	else
		cfg_file_name = "qdsl2zdt.cfg";

	ifstream cfg_file(cfg_file_name.c_str());
	if(!cfg_file)
	{	
		cout << "can't read" << cfg_file_name << "\n"; 
		exit(EXIT_FAILURE);
	}

	//read config
	string config_doc;
	cfg_file.seekg(0, ios_base::end);
	config_doc.resize(cfg_file.tellg());
	cfg_file.seekg(0, ios_base::beg);
	cfg_file.read(&config_doc[0], config_doc.size());

	Json::Value root;
	Json::Reader reader;
	bool parsingSuccessful = reader.parse(config_doc, root);
	if(!parsingSuccessful)
	{
		std::cout << "Failed to parse " << config_doc << " file\n" <<
				      reader.getFormatedErrorMessages();

		exit(EXIT_FAILURE);
	}

	//conversion
	clock_t t1 = clock();
	
	int count_words = 0;
	string line;
	int linenum = 0;
	QString word_part, translation_part;

	while(getline(infile, line))
	{
		if(line[0] == '#')//grep -v '^#'
			continue;

		if(line[line.size() - 1] == '\n')// chop \n
			line.resize(line.size() - 1);
		if(line.size() > 0 && line[line.size()-1] == '\r')
			line.resize(line.size() - 1);

		if(line == "")
			continue;

		if(line[0] != '\t')
		{
			if(word_part.length())
			{
				translation_part += "\n";					// appending \n

				do_replace(root["translation_part_replace"], translation_part);

				outfile << ((const char *)word_part.toAscii()) << ((const char *)translation_part.toAscii());	// write
				translation_part.resize(0);	// freeing the buffer
			}
			
			word_part = line.c_str();
			do_replace(root["word_part_replace"], word_part);
			word_part += "  ";
			++count_words;
		}
		else
		{
			//if(line.length())
			{
				QString qline = line.c_str();

				do_replace(root["dsl_translation_line_replace"], qline);

				translation_part += qline;
			}
		}

		


		/*if (line[0] != '\t')
		{
			if(line[line.size() - 1] == '\n')// chop \n
				line.resize(line.size() - 1);

			if(line.size() > 0 && line[line.size()-1] == '\r')
				line.resize(line.size() - 1);

			QString qline = line.c_str();

			if(!linenum)		// separate a "word" from his article
			{
				word_part = qline;
				do_replace(root["word_part_replace"], word_part);
				word_part += "  ";
				++count_words;
			}
			else
			{
				if(linenum == 1)
					do_replace(root["dsl_translation_firstline_replace"], qline);
				else
					do_replace(root["dsl_translation_line_replace"], qline);

				translation_part += qline;
			}
		}
		else
		{
			translation_part += "\n";					// appending \n

			do_replace(root["translation_part_replace"], translation_part);

			outfile << ((const char *)word_part.toAscii()) << ((const char *)translation_part.toAscii());	// write
			translation_part.resize(0);	// freeing the buffer
			linenum = -1;				// next line is first in dict article
		}

		++linenum;*/
	}

	clock_t t2 = clock();

	cout.setf(ios::fixed, ios::floatfield);
	cout.precision(2);
	cout << "Done (processed " << count_words << " words, " << (double(t2-t1) / CLOCKS_PER_SEC) << " sec)";

	return 0;
}
